@@ -24,10 +24,12 @@
 #include <config.h>
 #include <vpn.h>
 #include <autoopts/options.h>
+#include <ipc.pb-c.h>
 
 int add_multi_line_val(void *pool, const char *name, char ***s_name, size_t *num,
 	 	       tOptionValue const *pov,
 		       const tOptionValue *val);
+int cfg_parse_ports(void *pool, FwPortSt ***fw_ports, size_t *n_fw_ports, const char *str);
 
 #define MAX_SUBOPTIONS 5
 
