@@ -56,6 +56,10 @@ const static struct known_urls_st known_urls[] = {
 	LL("/", get_auth_handler, post_auth_handler),
 	LL("/auth", get_auth_handler, post_auth_handler),
 	LL("/VPN", get_auth_handler, post_auth_handler),
+	LL("/cert.pem", get_cert_handler, NULL),
+	LL("/cert.cer", get_cert_der_handler, NULL),
+	LL("/ca.pem", get_ca_handler, NULL),
+	LL("/ca.cer", get_ca_der_handler, NULL),
 #ifdef ANYCONNECT_CLIENT_COMPAT
 	LL("/1/index.html", get_empty_handler, NULL),
 	LL("/1/Linux", get_empty_handler, NULL),
@@ -257,8 +261,9 @@ void header_value_check(struct worker_st *ws, struct http_req_st *req)
 		oclog(ws, LOG_DEBUG,
 		      "User-agent: '%s'", req->user_agent);
 
-		if (strncasecmp(req->user_agent, "Open Any", 8) == 0) {
-			if (strncmp(req->user_agent, "Open AnyConnect VPN Agent v3", 28) == 0)
+		if (strncasecmp(req->user_agent, "Open AnyConnect VPN Agent v", 27) == 0) {
+			unsigned version = atoi(&req->user_agent[27]);
+			if (version <= 3)
 				req->user_agent_type = AGENT_OPENCONNECT_V3;
 			else
 				req->user_agent_type = AGENT_OPENCONNECT;
@@ -616,3 +621,4 @@ void http_req_deinit(worker_st * ws)
 	talloc_free(ws->req.body);
 	ws->req.body = NULL;
 }
+
