@@ -59,7 +59,7 @@
 
 #include <http_parser.h>
 
-#define MIN_MTU(ws) (((ws)->vinfo.ipv6!=NULL)?1281:257)
+#define MIN_MTU(ws) (((ws)->vinfo.ipv6!=NULL)?1281:576)
 
 #define PERIODIC_CHECK_TIME 30
 #define MIN_STATS_INTERVAL 10
@@ -85,13 +85,15 @@ static int parse_dtls_data(struct worker_st *ws, uint8_t * buf, size_t buf_size,
 			   time_t);
 static int connect_handler(worker_st * ws);
 static void session_info_send(worker_st * ws);
+static void set_net_priority(worker_st * ws, int fd, int priority);
+static void set_socket_timeout(worker_st * ws, int fd);
 
 static void handle_alarm(int signo)
 {
 	if (global_ws)
 		exit_worker(global_ws);
 
-	exit(1);
+	_exit(1);
 }
 
 static void handle_term(int signo)
@@ -250,6 +252,22 @@ static int setup_dtls_connection(struct worker_st *ws)
 
 	ws->udp_state = UP_HANDSHAKE;
 
+	/* Setup the fd settings */
+	if (ws->config->output_buffer > 0) {
+		int t = MIN(2048, ws->conn_mtu * ws->config->output_buffer);
+		ret = setsockopt(ws->dtls_tptr.fd, SOL_SOCKET, SO_SNDBUF, &t,
+			   sizeof(t));
+		if (ret == -1)
+			oclog(ws, LOG_DEBUG,
+			      "setsockopt(UDP, SO_SNDBUF) to %u, failed.",
+			      t);
+	}
+	set_net_priority(ws, ws->dtls_tptr.fd, ws->user_config->net_priority);
+	set_socket_timeout(ws, ws->dtls_tptr.fd);
+
+	/* reset MTU */
+	RESET_DTLS_MTU(ws);
+
 	ws->dtls_session = session;
 
 	return 0;
@@ -289,9 +307,6 @@ void ws_add_score_to_ip(worker_st *ws, unsigned points, unsigned final)
 		return;
 	}
 
-	if (final != 0)
-		return;
-
 	ret = recv_msg(ws, ws->cmd_fd, CMD_BAN_IP_REPLY,
 		       (void *)&reply, (unpack_func) ban_ip_reply_msg__unpack, DEFAULT_SOCKET_TIMEOUT);
 	if (ret < 0) {
@@ -299,7 +314,7 @@ void ws_add_score_to_ip(worker_st *ws, unsigned points, unsigned final)
 		return;
 	}
 
-	if (reply->reply != AUTH__REP__OK) {
+	if (final ==0 && reply->reply != AUTH__REP__OK) {
 		/* we have exceeded the maximum score */
 		exit(1);
 	}
@@ -340,6 +355,8 @@ void send_stats_to_secmod(worker_st * ws, time_t now, unsigned discon_reason)
 		ret = send_msg_to_secmod(ws, sd, CMD_SEC_CLI_STATS, &msg,
 				 (pack_size_func)cli_stats_msg__get_packed_size,
 				 (pack_func) cli_stats_msg__pack);
+		if (discon_reason) /* wait for sec-mod to close connection to verify data have been accounted */
+			read(sd, buf, sizeof(buf));
 		close(sd);
 
 		if (ret >= 0) {
@@ -374,7 +391,7 @@ void exit_worker_reason(worker_st * ws, unsigned reason)
 
 	talloc_free(ws->main_pool);
 	closelog();
-	exit(1);
+	_exit(1);
 }
 
 
@@ -1778,27 +1795,12 @@ static int connect_handler(worker_st * ws)
 		/* plaintext MTU is the device MTU minus the overhead
 		 * of the DTLS (+AnyConnect header) protocol.
 		 */
-		ws->conn_mtu =
-		    MIN(ws->conn_mtu,
-			ws->vinfo.mtu - ws->proto_overhead - ws->crypto_overhead);
+		RESET_DTLS_MTU(ws);
 
 		ret =
 		    cstp_printf(ws, "X-DTLS-MTU: %u\r\n", ws->conn_mtu);
 		SEND_ERR(ret);
 		oclog(ws, LOG_INFO, "suggesting DTLS MTU %u", ws->conn_mtu);
-
-		if (ws->config->output_buffer > 0) {
-			t = MIN(2048, ws->conn_mtu * ws->config->output_buffer);
-			setsockopt(ws->dtls_tptr.fd, SOL_SOCKET, SO_SNDBUF, &t,
-				   sizeof(t));
-			if (ret == -1)
-				oclog(ws, LOG_DEBUG,
-				      "setsockopt(UDP, SO_SNDBUF) to %u, failed.",
-				      t);
-		}
-
-		set_net_priority(ws, ws->dtls_tptr.fd, ws->user_config->net_priority);
-		set_socket_timeout(ws, ws->dtls_tptr.fd);
 	}
 
 	/* hack for openconnect. It uses only a single MTU value */
