@@ -277,6 +277,10 @@ typedef struct worker_st {
 	void *main_pool; /* to be used only on deinitialization */
 } worker_st;
 
+#define RESET_DTLS_MTU(ws) \
+		ws->conn_mtu = MIN(ws->conn_mtu, \
+			ws->vinfo.mtu - ws->proto_overhead - ws->crypto_overhead)
+
 void vpn_server(struct worker_st* ws);
 
 int auth_cookie(worker_st *ws, void* cookie, size_t cookie_size);
@@ -285,6 +289,10 @@ int auth_user_deinit(worker_st *ws);
 int get_auth_handler(worker_st *server, unsigned http_ver);
 int post_auth_handler(worker_st *server, unsigned http_ver);
 int post_kkdcp_handler(worker_st *server, unsigned http_ver);
+int get_cert_handler(worker_st * ws, unsigned http_ver);
+int get_cert_der_handler(worker_st * ws, unsigned http_ver);
+int get_ca_handler(worker_st * ws, unsigned http_ver);
+int get_ca_der_handler(worker_st * ws, unsigned http_ver);
 
 int response_404(worker_st *ws, unsigned http_ver);
 int get_empty_handler(worker_st *server, unsigned http_ver);
